#include "characterlcd.h"
 
//******************************************************************************
/// Configures LCD Pins as Output
void InitLcdPins(void)
{
    /// Pin direction configuration (output)
    SetPinAltFn(D4_PIN, -1, DIR_OUT);     ///< D4
    SetPinAltFn(D5_PIN, -1, DIR_OUT);     ///< D5
    SetPinAltFn(D6_PIN, -1, DIR_OUT);     ///< D6
    SetPinAltFn(D7_PIN, -1, DIR_OUT);     ///< D7
    SetPinAltFn(RS_PIN, -1, DIR_OUT);     ///< RS
    SetPinAltFn(EN_PIN, -1, DIR_OUT);     ///< EN
 
}
 
//******************************************************************************
/// Initializes character LCD for 4 bit mode
void InitLcd(void)
{
    BYTE tmp = 0;
    BYTE lcdInitCommand[] = { LCD_INIT_COMMAND_1,
                              LCD_INIT_COMMAND_2,
                              LCD_INIT_COMMAND_3,
                              LCD_INIT_COMMAND_4,
                              LCD_INIT_COMMAND_5,
                              LCD_INIT_COMMAND_6,
                              LCD_INIT_COMMAND_7 };

    CLRBIT(D7_PIN);
    CLRBIT(D6_PIN);
    SETBIT(D5_PIN);
    SETBIT(D4_PIN);
    SETBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
    CLRBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
 
    CLRBIT(D7_PIN);
    CLRBIT(D6_PIN);
    SETBIT(D5_PIN);
    SETBIT(D4_PIN);
    SETBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
    CLRBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
 
    CLRBIT(D7_PIN);
    CLRBIT(D6_PIN);
    SETBIT(D5_PIN);
    SETBIT(D4_PIN);
    SETBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
    CLRBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
 
    CLRBIT(D7_PIN);
    CLRBIT(D6_PIN);
    SETBIT(D5_PIN);
    CLRBIT(D4_PIN);
    SETBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
    CLRBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
 
    /// Send LCD initialization commands
    for (tmp = 0; tmp < 7; tmp++)
    {
        SendByteToLcd(lcdInitCommand[tmp], COMMAND);
        Sleep(LCD_INIT_DELAY);
    }
}
 
//******************************************************************************
/// Sends data/command to character LCD
/// @param[in]     rawByte  Byte to send 
/// @param[in]     type     COMMNAD or DATA
void SendByteToLcd(BYTE rawByte, BOOL type)
{
    /// COMMAND or DATA selection (RS)
    if (type == TRUE) SETBIT(RS_PIN);
    else CLRBIT(RS_PIN);
 
    /// Higher nibble
    if (rawByte & 0x10) SETBIT(D4_PIN);
    else CLRBIT(D4_PIN);
 
    if (rawByte & 0x20) SETBIT(D5_PIN);
    else CLRBIT(D5_PIN);
 
    if (rawByte & 0x40) SETBIT(D6_PIN);
    else CLRBIT(D6_PIN);
 
    if (rawByte & 0x80) SETBIT(D7_PIN);
    else CLRBIT(D7_PIN);
 
    SETBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
    CLRBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
 
    /// Lower nibble
    if (rawByte & 0x01) SETBIT(D4_PIN);
    else CLRBIT(D4_PIN);
 
    if (rawByte & 0x02) SETBIT(D5_PIN);
    else CLRBIT(D5_PIN);
 
    if (rawByte & 0x04) SETBIT(D6_PIN);
    else CLRBIT(D6_PIN);
 
    if (rawByte & 0x08) SETBIT(D7_PIN);
    else CLRBIT(D7_PIN);
 
    SETBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
    CLRBIT(EN_PIN);
    Sleep(LCD_EN_DELAY);
}
//******************************************************************************
/// Display character on LCD
/// @param[in]     lineNumber           Line on which character to be printed
/// @param[in]     characterPosition    Position of character
/// @param[in]     character            Character to be printed 
void PutCharacterOnLcd(BYTE lineNumber, BYTE characterPosition, char character)
{
    SendByteToLcd(lineNumber + characterPosition, COMMAND);     ///< Set cursor position
    SendByteToLcd(character, DATA);                             ///< Send data
}
 
//******************************************************************************
/// Display string on character LCD
/// @param[in]     lineNumber           Line on which string to be printed
/// @param[in]     stringStartPosition  Start position of the string
/// @param[in]     string               String to be displayed
/// @param[in]     stringLength         Length of the string
void PutStringOnLcd(BYTE lineNumber, BYTE stringStartPosition, char *string, DWORD stringLength)
{
    SendByteToLcd(lineNumber + stringStartPosition, COMMAND);   ///< Set cursor position
    while ((*string) && (stringLength > 0))
    {
        SendByteToLcd(*string, DATA);                           ///< Send data
        string++;
        stringLength--;
    }
}