#ifndef _CHARACTERLCD_H_
#define _CHARACTERLCD_H_
 
#include "gpioLib.h"
 
/// LCD configuration
#define NO_OF_LINE                  4           ///< 4 line character LCD
#define NO_OF_CHARACTER_PER_LINE    20          ///< 20 characters per line

/// LCD initialization commands for 4-bit mode
#define LCD_INIT_COMMAND_1          0x20        ///< 4-bit interface
#define LCD_INIT_COMMAND_2          0x20        ///< 4-bit interface
#define LCD_INIT_COMMAND_3          0x20        ///< 4-bit interface
#define LCD_INIT_COMMAND_4          0x28        ///< 4-bit, 2 line
#define LCD_INIT_COMMAND_5          0x0C        ///< Display ON, cursor OFF
#define LCD_INIT_COMMAND_6          0x06        ///< Entry mode - no shift, auto increment
#define LCD_INIT_COMMAND_7          0x01        ///< Clear LCD

/// LCD pin connections for 4-bit mode
/// Please configure these macro as per your connections
#define D4_PIN                      85          ///< Iris connector X16 pin 18, used as GPIO
#define D5_PIN                      97          ///< Iris connector X16 pin 17, used as GPIO
#define D6_PIN                      101         ///< Iris connector X16 pin 16, used as GPIO
#define D7_PIN                      103         ///< Iris connector X16 pin 15, used as GPIO

#define EN_PIN                      79          ///< Iris connector X16 pin 19, used as GPIO
#define RS_PIN                      45          ///< Iris connector X16 pin 20, used as GPIO

#define LCD_INIT_DELAY              50          ///< Delay in milliseconds to be used in LCD initialization
#define LCD_EN_DELAY                1           ///< Delay in milliseconds to be used in LCD data/command send

#define COMMAND                     0           ///< For command RS Pin should be low
#define DATA                        1           ///< For data RS Pin should be high

#define LINE_1                      0x80        ///< Starting address of line 1
#define LINE_2                      0xC0        ///< Starting address of line 2 
#define LINE_3                      0x94        ///< Starting address of line 3 
#define LINE_4                      0xD4        ///< Starting address of line 4 

#define CLRBIT(PIN)                 SetPinLevel(PIN, FALSE)
#define SETBIT(PIN)                 SetPinLevel(PIN, TRUE)
 
//******************************************************************************
/// Configures LCD Pins as Output
void InitLcdPins(void);
 
//******************************************************************************
/// Initializes character LCD for 4 bit mode
void InitLcd(void);
 
//******************************************************************************
/// Sends data/command to character LCD
/// @param[in]     rawByte  Byte to send 
/// @param[in]     type     COMMNAD or DATA
void SendByteToLcd(BYTE rawByte, BOOL type);
 
//******************************************************************************
/// Display character on LCD
/// @param[in]     lineNumber           Line on which character to be printed
/// @param[in]     characterPosition    Position of character
/// @param[in]     character            Character to be displayed
void PutCharacterOnLcd(BYTE lineNumber, BYTE characterPosition, char character);
 
//******************************************************************************
/// Display string on character LCD
/// @param[in]     lineNumber           Line on which string to be printed
/// @param[in]     stringStartPosition  Start position of the string
/// @param[in]     string               String to be displayed
/// @param[in]     stringLength         Length of the string
void PutStringOnLcd(BYTE lineNumber, BYTE stringStartPosition, char *string, DWORD stringLength);

#endif 